unit SDCCShellExt_U;
// This file is part of Sarah Dean's Code Co-op Shell Extension.
//
//  "Sarah Dean's Code Co-op Shell Extension" is free software; you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation; either
// version 2 of the License, or at your option) any later version.
//
//  "Sarah Dean's Code Co-op Shell Extension" is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with "Sarah Dean's Code Co-op Shell Extension"; if not, write to
// the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
// MA  02111-1307  USA


// Description: Sarah Dean's Code Co-op Shell Extension
// By Sarah Dean
//
// Shell Extension for Code Co-op
//
// -----------------------------------------------------------------------------


interface

uses
  Windows, ActiveX, ComObj, ShlObj, Classes,
  CodeCoop,
  SDCCIniFile_U,
  frmProgress_U;

type
  TInternalOp = (opAddItems, opCheckin, opCheckout, opUncheckout, internalBackup);

type
  TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
  private
    SDCC_INI_FILE: TSDCCIniFile;
    CODE_COOP_OBJ: TCodeCoop;

    bhSubmenu: HBitMap;
    bhAddItem: HBitMap;
    bhCheckin: HBitMap;
    bhCheckout: HBitMap;
    bhUncheckout: HBitMap;

    FFileName: array of array [0..MAX_PATH] of Char;
  protected
    { IShellExtInit }
    function IShellExtInit.Initialize = SEIInitialize; // Avoid compiler warning
    function SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
      hKeyProgID: HKEY): HResult; stdcall;
    { IContextMenu }
    function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
      uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;

    function  DeepCountItemsUnder(item: string): integer;

    procedure ProcessItems(operation: TInternalOp);

    function  DoActualProcessSingleItem(
                                        operation: TInternalOp;
                                        singleItem: string; recursive: boolean; dlgProgress: TfrmProgress;
                                        // Only valid for opAdd
                                        fileType: TCCFileType;
                                        // Only valid for opUncheckout
                                        makeBackup: boolean;
                                        // Only valid for opCheckin
                                        comment: string; keepCheckedOut: boolean
                                       ): boolean;

    function  DoActualProcessItems(
                                   operation: TInternalOp;
                                   items: TStringList; recursive: boolean; dlgProgress: TfrmProgress;
                                   // Only valid for opAdd
                                   fileType: TCCFileType;
                                   // Only valid for opUncheckout
                                   makeBackup: boolean;
                                   // Only valid for opCheckin
                                   comment: string; keepCheckedOut: boolean
                                  ): boolean;

    function  FileBackup(srcFilename: string): boolean;
    function  GetBackupFilename(srcFilename: string): string;
    function  FileCopy(srcFilename, destFilename: string): boolean;

  public
    destructor Destroy(); override;
  end;

const
  MENUOFFSET_ADDFILE     = 1;
  MENUOFFSET_CHECKIN     = 2;
  MENUOFFSET_CHECKOUT    = 3;
  MENUOFFSET_UNCHECKOUT  = 4;

  MAX_NUMBER_MENUITEMS_ADDED = 5; // This MUST always be set to the highest
                                  // number + 1

implementation

{$R MenuBitmaps.res}

uses
  Controls, // Required for "mrOK"
  ComServ, SysUtils, ShellApi, Registry, RegStr, FileCtrl,
  Dialogs, Forms,
  AppGlobalConsts_U,
  SDUGeneral,
  SDUFileIterator_U,
  frmAddFiles_U,
  frmCheckin_U,
  frmCheckout_U,
  frmUncheckout_U;



destructor TContextMenu.Destroy();
begin
  if (bhSubmenu<>0) then
    begin
    DeleteObject(bhSubmenu);
    end;

  if (bhAddItem<>0) then
    begin
    DeleteObject(bhAddItem);
    end;

  if (bhCheckin<>0) then
    begin
    DeleteObject(bhCheckin);
    end;

  if (bhCheckout<>0) then
    begin
    DeleteObject(bhCheckout);
    end;

  if (bhUncheckout<>0) then
    begin
    DeleteObject(bhUncheckout);
    end;


  if (CODE_COOP_OBJ<>nil) then
    begin
    CODE_COOP_OBJ.Free();
    end;

  if (SDCC_INI_FILE<>nil) then
    begin
    SDCC_INI_FILE.Free();
    end;

  inherited;
end;


// Perform initialization
function TContextMenu.SEIInitialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  StgMedium: TStgMedium;
  FormatEtc: TFormatEtc;
  i: integer;
begin
  // Fail the call if lpdobj is Nil.
  if (lpdobj = nil) then begin
    Result := E_INVALIDARG;
    Exit;
  end;

  with FormatEtc do begin
    cfFormat := CF_HDROP;
    ptd      := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex   := -1;
    tymed    := TYMED_HGLOBAL;
  end;


  if (CODE_COOP_OBJ = nil) then
    begin
    CODE_COOP_OBJ := TCodeCoop.Create();
    end;

  CODE_COOP_OBJ.Active := TRUE;
  if not(CODE_COOP_OBJ.Active) then
    begin
    CODE_COOP_OBJ.Free();
    Result := E_FAIL;
    exit;
    end;

  if (SDCC_INI_FILE = nil) then
    begin
    SDCC_INI_FILE := TSDCCIniFile.Create(INI_FILENAME);
    // Read in the ini file/setup it's defaults.
    SDCC_INI_FILE.ReadIniFile();
    end;


  bhSubmenu:= 0;
  bhAddItem:= 0;
  bhCheckin:= 0;
  bhCheckout:= 0;
  bhUncheckout:= 0;


  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := lpdobj.GetData(FormatEtc, StgMedium);
  if Failed(Result) then
    Exit;
  // If only one file is selected, retrieve the file name and store it in
  // FFileName. Otherwise fail the call.
  SetLength(FFilename, 0);
  SetLength(FFilename, DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0));
  for i:=low(FFilename) to high(FFilename) do
    begin
    DragQueryFile(StgMedium.hGlobal, i, FFileName[i], SizeOf(FFileName[i]));
    end;
  Result := NOERROR;

  ReleaseStgMedium(StgMedium);
end;


// Function called by the shell when it wants us to add our menuitems
function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
          idCmdLast, uFlags: UINT): HResult;
var
  menuIndexOffset: UINT;
  i, j: integer;

  addMenuItemsMenu: HMENU;
  addMenuItemsMenuIndex: UINT;
  firstStr: string;

  fileStatus: TCCFileStatus;

  menuItemAddFile: boolean;
  menuItemCheckin: boolean;
  menuItemCheckout: boolean;
  menuItemUncheckout: boolean;

  projectRootDirs: TStringList;
  addCodeCoopMenuIems: boolean;
begin
  Result := MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, 0);
  menuIndexOffset := 0;

  firstStr := FFilename[low(FFilename)];

  if ((uFlags and $0000000F) = CMF_NORMAL) or
     ((uFlags and CMF_EXPLORE) <> 0) then
    begin
    // Determine if any menuitems should be added at all...
    addCodeCoopMenuIems := FALSE;
    for i:=low(FFilename) to high(FFilename) do
      begin
      fileStatus := CODE_COOP_OBJ.FileStatus(FFilename[i]);
      if (fileStatus <> fileStatusUnknown) then
        begin
        addCodeCoopMenuIems := TRUE;
        break;
        end;
      end;


    if (addCodeCoopMenuIems) then
      begin
      if (CODE_COOP_OBJ.Active) then
        begin
        if SDCC_INI_FILE.OptShellExtMenuSeparators then
          begin
          InsertMenu(Menu,
                     indexMenu + menuIndexOffset,
                     MF_SEPARATOR or MF_BYPOSITION,
                     0,
                     nil);
          inc(menuIndexOffset);
          end;


        if not(SDCC_INI_FILE.OptShellExtSubMenu) then
          begin
          addMenuItemsMenu := Menu;
          addMenuItemsMenuIndex := indexMenu + menuIndexOffset;
          end
        else
          begin
          addMenuItemsMenu := CreatePopupMenu();
          InsertMenu(Menu,
                     indexMenu+menuIndexOffset,
                     MF_POPUP or MF_BYPOSITION,
                     addMenuItemsMenu,
                     PChar(SDCC_INI_FILE.OptShellExtSubMenuTitle));
          bhSubmenu:=LoadBitmap(hinstance, PChar('SUBMENU'));
          if (bhSubmenu<>0) then
            begin
            SetMenuItemBitmaps(
                               Menu,
                               indexMenu+menuIndexOffset,
                               MF_BYPOSITION,
                               bhSubmenu,
                               bhSubmenu
                              );
            end;
          addMenuItemsMenuIndex := 0;
          inc(menuIndexOffset);
          end;

        menuItemAddFile:= FALSE;
        menuItemCheckin:= FALSE;
        menuItemCheckout:= FALSE;
        menuItemUncheckout:= FALSE;

        projectRootDirs:= TStringList.Create();
        try
          CODE_COOP_OBJ.GetProjects(nil, projectRootDirs);
          for i:=0 to (projectRootDirs.count-1) do
            begin
            projectRootDirs[i] := uppercase(projectRootDirs[i]);
            end;


          for i:=low(FFilename) to high(FFilename) do
            begin
            fileStatus := CODE_COOP_OBJ.FileStatus(FFilename[i]);
            if (fileStatus = fileStatusNotUnderSCC) then
              begin
              // Check if file is under one of the Code Co-op project dirs
              for j:=0 to (projectRootDirs.count-1) do
                begin
                if (Pos(projectRootDirs[j], uppercase(FFilename[i]))>0) then
                  begin
                  menuItemAddFile := TRUE;
                  break;
                  end;
                end;

              end
            else if (fileStatus = fileStatusCheckedIn) then
              begin
              menuItemCheckout := TRUE;
              end
            else if (fileStatus = fileStatusCheckedOut) then
              begin
              menuItemCheckin := TRUE;
              menuItemUncheckout := TRUE;
              end;

            end;  // for i:=low(FFilename) to high(FFilename) do

        finally
          projectRootDirs.Free();
        end;

        if menuItemAddFile then
          begin
          InsertMenu(addMenuItemsMenu,
                     addMenuItemsMenuIndex,
                     MF_STRING or MF_BYPOSITION,
                     idCmdFirst+MENUOFFSET_ADDFILE,
                     'Add to Code Co-op...');
          bhAddItem:=LoadBitmap(hinstance, PChar('ADDITEM'));
          if (bhAddItem<>0) then
            begin
            SetMenuItemBitmaps(
                               addMenuItemsMenu,
                               addMenuItemsMenuIndex,
                               MF_BYPOSITION,
                               bhAddItem,
                               bhAddItem
                              );
            end;

          inc(addMenuItemsMenuIndex);
          end
        else
          begin
          if menuItemCheckout then
            begin
            InsertMenu(addMenuItemsMenu,
                       addMenuItemsMenuIndex,
                       MF_STRING or MF_BYPOSITION,
                       idCmdFirst+MENUOFFSET_CHECKOUT,
                       'Checkout...');
            bhCheckout:=LoadBitmap(hinstance, PChar('CHECKOUT'));
            if (bhCheckout<>0) then
              begin
              SetMenuItemBitmaps(
                                 addMenuItemsMenu,
                                 addMenuItemsMenuIndex,
                                 MF_BYPOSITION,
                                 bhCheckout,
                                 bhCheckout
                                );
              end;

            inc(addMenuItemsMenuIndex);
            end;

          if menuItemCheckin then
            begin
            InsertMenu(addMenuItemsMenu,
                       addMenuItemsMenuIndex,
                       MF_STRING or MF_BYPOSITION,
                       idCmdFirst+MENUOFFSET_CHECKIN,
                       'Checkin...');
            bhCheckin:=LoadBitmap(hinstance, PChar('CHECKIN'));
            if (bhCheckin<>0) then
              begin
              SetMenuItemBitmaps(
                                 addMenuItemsMenu,
                                 addMenuItemsMenuIndex,
                                 MF_BYPOSITION,
                                 bhCheckin,
                                 bhCheckin
                                );
              end;

            inc(addMenuItemsMenuIndex);
            end;

          if menuItemUncheckout then
            begin
            InsertMenu(addMenuItemsMenu,
                       addMenuItemsMenuIndex,
                       MF_STRING or MF_BYPOSITION,
                       idCmdFirst+MENUOFFSET_UNCHECKOUT,
                       'Uncheckout...');
            bhUncheckout:=LoadBitmap(hinstance, PChar('UNCHECKOUT'));
            if (bhUncheckout<>0) then
              begin
              SetMenuItemBitmaps(
                                 addMenuItemsMenu,
                                 addMenuItemsMenuIndex,
                                 MF_BYPOSITION,
                                 bhUncheckout,
                                 bhUncheckout
                                );
              end;

            inc(addMenuItemsMenuIndex);
            end;

          end;


        if SDCC_INI_FILE.OptShellExtMenuSeparators then
          begin
          if SDCC_INI_FILE.OptShellExtSubMenu then
            begin
            addMenuItemsMenuIndex := menuIndexOffset;
            end;

          InsertMenu(Menu,
                     addMenuItemsMenuIndex,
                     MF_SEPARATOR or MF_BYPOSITION,
                     0,
                     nil);
          end;


        end;  // if (SDCC_INI_FILE.OptShellExtCodeCoopBaseDir<>'') then

      // Return number of menu items added
      // xxx - lplp - IS THIS CORRECT?!
      Result := MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, MAX_NUMBER_MENUITEMS_ADDED)
  //      Result := MakeResult(SEVERITY_SUCCESS, FACILITY_NULL, addMenuItemsMenuIndex)

      end;
    end;

end;


// Function called by the shell to invoke one of out menuitems
function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
begin
  // Make sure we are not being called by an application
  if (HiWord(Integer(lpici.lpVerb)) <> 0) then
    begin
    Result := E_FAIL;
    Exit;
    end;


  if not(CODE_COOP_OBJ.Active) then
    begin
    MessageDlg(
               'You must configure the  location of the Code Co-op'+CRLF+
               'executables before using the Code Co-op shell extension.',
               mtError,
               [mbOK],
               0
              );
    Result := E_FAIL;
    exit;
    end
  else
    begin
    if (LoWord(lpici.lpVerb) = MENUOFFSET_ADDFILE) then
      begin
      ProcessItems(opAddItems);
      end
    else if (LoWord(lpici.lpVerb) = MENUOFFSET_CHECKIN) then
      begin
      ProcessItems(opCheckin);
      end
    else if (LoWord(lpici.lpVerb) = MENUOFFSET_CHECKOUT) then
      begin
      ProcessItems(opCheckout);
      end
    else if (LoWord(lpici.lpVerb) = MENUOFFSET_UNCHECKOUT) then
      begin
      ProcessItems(opUncheckout);
      end
    else
      begin
      Result := E_INVALIDARG;
      Exit;
      end;

    end;

  Result := NOERROR;

end;


// Function called by the shell to obtain status (help) text, etc
function TContextMenu.GetCommandString(idCmd, uType: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HRESULT;
begin
  if (idCmd = MENUOFFSET_ADDFILE) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Add items to Code Co-op');
      end;
    Result := NOERROR;
    end
  else if (idCmd = MENUOFFSET_CHECKIN) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Check items into Code Co-op');
      end;
    Result := NOERROR;
    end
  else if (idCmd = MENUOFFSET_CHECKOUT) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Check items out of Code Co-op');
      end;
    Result := NOERROR;
    end
  else if (idCmd = MENUOFFSET_UNCHECKOUT) then
    begin
    if (uType = GCS_HELPTEXT) then
      begin
      // return help string for menu item
      StrCopy(pszName, 'Uncheck items out of Code Co-op');
      end;
    Result := NOERROR;
    end
  else
    begin
    Result := E_INVALIDARG;
    end;
end;



procedure TContextMenu.ProcessItems(operation: TInternalOp);
var
  dlgPrompt: TForm;
  dlgProgress: TfrmProgress;
  i: integer;
  fileType: TCCFileType;
  itemsLst: TStringList;
  recursive: boolean;
  checkInAfterAdd: boolean;
  makeBackup: boolean;
  comment: string;
  keepCheckedOut: boolean;
  goAhead: boolean;
  includesSubdirs: boolean;
begin
  includesSubdirs := FALSE;
  
  itemsLst:=TStringList.Create();
  try
    for i:=low(FFilename) to high(FFilename) do
      begin
      itemsLst.Add(FFilename[i]);
      end;

    dlgPrompt := nil;
    case operation of
      opAddItems:
        begin
        dlgPrompt := TfrmAddFiles.Create(nil);
        TfrmAddFiles(dlgPrompt).reFiles.lines.AddStrings(itemsLst);
        end;

      opCheckin:
        begin
        dlgPrompt := TfrmCheckin.Create(nil);
        TfrmCheckin(dlgPrompt).reFiles.lines.AddStrings(itemsLst);
        end;

      opCheckout:
        begin
        includesSubdirs := FALSE;
        for i:=0 to (itemsLst.count-1) do
          begin
          if DirectoryExists(itemsLst[i]) then
            begin
            includesSubdirs := TRUE;
            break;
            end;
          end;

        dlgPrompt := TfrmCheckout.Create(nil);
        TfrmCheckout(dlgPrompt).IncludesSubdirs := includesSubdirs;
        TfrmCheckout(dlgPrompt).reFiles.lines.AddStrings(itemsLst);
        end;

      opUncheckout:
        begin
        dlgPrompt := TfrmUncheckout.Create(nil);
        TfrmUncheckout(dlgPrompt).reFiles.lines.AddStrings(itemsLst);
        end;

    end;

    try
      // Special case - don't confirm checkouts if there's no subdirs,
      // and the user doesn't want to confirm them
      if (
          (operation = opCheckout) and
          not(includesSubdirs) and
          not(SDCC_INI_FILE.OptShellExtConfirmCheckouts)
         )  then
        begin
        goAhead := TRUE;
        end
      else
        begin
        goAhead := (dlgPrompt.ShowModal() = mrOK);
        end;


      if (goAhead) then
        begin
        recursive := FALSE;
        fileType := fileTypeBinary;
        makeBackup := FALSE;
        comment := '';
        keepCheckedOut:= FALSE;
        checkInAfterAdd:= FALSE;
        
        case operation of
          opAddItems:
            begin
            recursive := TfrmAddFiles(dlgPrompt).ckRecurseSubdirs.checked;
            checkInAfterAdd:= TfrmAddFiles(dlgPrompt).ckCheckinAfterAdd.checked;
            
            // Determine type of files being checked in
            if TfrmAddFiles(dlgPrompt).rbHeader.checked then
              begin
              fileType := fileTypeHeader;
              end
            else if TfrmAddFiles(dlgPrompt).rbSource.checked then
              begin
              fileType := fileTypeSource;
              end
            else if TfrmAddFiles(dlgPrompt).rbText.checked then
              begin
              fileType := fileTypeText;
              end
            else
              begin
              fileType := fileTypeBinary;
              end;

            end;

          opCheckin:
            begin
            recursive := TfrmCheckin(dlgPrompt).ckRecurseSubdirs.checked;
            comment := TfrmCheckin(dlgPrompt).reComments.Text;
            keepCheckedOut := TfrmCheckin(dlgPrompt).ckKeepCheckedOut.checked;
            end;

          opCheckout:
            begin
            recursive := TfrmCheckout(dlgPrompt).ckRecurseSubdirs.checked;
            end;

          opUncheckout:
            begin
            recursive := TfrmUncheckout(dlgPrompt).ckRecurseSubdirs.checked;
            makeBackup := TfrmUncheckout(dlgPrompt).ckMakeBackup.checked;
            end;

        end;


        dlgProgress:= TfrmProgress.Create(nil);
        try
          dlgProgress.Max := 0;
          for i:=0 to (itemsLst.count-1) do
            begin
            dlgProgress.Max := dlgProgress.Max + DeepCountItemsUnder(itemsLst[i]);
            end;

          // Pre processing...
          if not(dlgProgress.Cancel) and (operation=opUncheckout) and (makeBackup) then
            begin
            dlgProgress.Position := 0;
            dlgProgress.Show();

            dlgProgress.Cancel := not(DoActualProcessItems(
                                                           internalBackup,
                                                           itemsLst, recursive, dlgProgress,
                                                           // Only valid for opAdd
                                                           fileType,
                                                           // Only valid for opUncheckout
                                                           makeBackup,
                                                           // Only valid for opCheckin
                                                           comment, keepCheckedOut
                                                          ));
            dlgProgress.Hide();
            end;


          if not(dlgProgress.Cancel) then
            begin
            dlgProgress.Position := 0;
            dlgProgress.Show();

            dlgProgress.Cancel := not(DoActualProcessItems(
                                                           operation,
                                                           itemsLst, recursive, dlgProgress,
                                                           // Only valid for opAdd
                                                           fileType,
                                                           // Only valid for opUncheckout
                                                           makeBackup,
                                                           // Only valid for opCheckin
                                                           comment, keepCheckedOut
                                                          ));

            dlgProgress.Hide();
            end;


          // Post processing...
          if not(dlgProgress.Cancel) and (operation=opAddItems) and (checkInAfterAdd) then
            begin
            ProcessItems(opCheckin);
            end;

        finally
          dlgProgress.Free();
        end;

        end;

    finally
      dlgPrompt.Free();
    end;

  finally
    itemsLst.Free();
  end;


end;







function TContextMenu.DoActualProcessSingleItem(
                                                operation: TInternalOp;
                                                singleItem: string; recursive: boolean; dlgProgress: TfrmProgress;
                                                // Only valid for opAdd
                                                fileType: TCCFileType;
                                                // Only valid for opUncheckout
                                                makeBackup: boolean;
                                                // Only valid for opCheckin
                                                comment: string; keepCheckedOut: boolean
                                               ): boolean;
var
  allOK: boolean;
  skipItem: boolean;
  increment: integer;
  currItemStatus: TCCFileStatus;
  errMsg: string;
  fileIterator: TSDUFileIterator;
  tmpFilename: string;
begin
  allOK:= TRUE;


  increment := 1;

  dlgProgress.Filename(singleItem);

  errMsg := '';
  currItemStatus := CODE_COOP_OBJ.FileStatus(singleItem);
  if (currItemStatus=fileStatusUnknown) then
    begin
    errMsg := 'This file''s current status could not be determined';
    end
  else
    begin
    // Call function on the operation
    case operation of
      opAddItems:
        begin
        if (currItemStatus=fileStatusCheckedIn) or (currItemStatus=fileStatusCheckedOut) then
          begin
          // Ignore - already under Code Co-op
          end
        else
          begin
          if not(CODE_COOP_OBJ.AddFile(singleItem, fileType)) then
            begin
            errMsg := 'This item could not be added to Code Co-op';
            end;
          end;

        end;

      opCheckin:
        begin
        if (currItemStatus=fileStatusCheckedIn) or (currItemStatus=fileStatusNotUnderSCC) then
          begin
          // Ignore - already checked in
          end
        else
          begin
          if not(CODE_COOP_OBJ.Checkin(singleItem, keepCheckedOut, comment)) then
            begin
            errMsg := 'This item could not be checked into Code Co-op';
            end;
          end;

        end;


      opCheckout:
        begin
        if (currItemStatus=fileStatusCheckedOut) or (currItemStatus=fileStatusNotUnderSCC) then
          begin
          // Ignore - already checked out
          end
        else
          begin
          if not(CODE_COOP_OBJ.Checkout(singleItem)) then
            begin
            errMsg := 'This item could not be checked out of Code Co-op';
            end;
          end;

        end;

      opUncheckout:
        begin
        if (currItemStatus=fileStatusCheckedIn) or (currItemStatus=fileStatusNotUnderSCC) then
          begin
          // Ignore - already checked in/not under Code Co-op
          end
        else
          begin
          if not(CODE_COOP_OBJ.Uncheckout(singleItem)) then
            begin
            errMsg := 'This item could not be uncheckedout of Code Co-op';
            end;
          end;

        end;

      // Pre-uncheckout process
      // Note: This only operates on files which have been checked out
      internalBackup:
        begin
        if DirectoryExists(singleItem) then
          begin
          // Ignore - don't backup directories en-masse
          end
        else
          begin
          if (currItemStatus=fileStatusCheckedIn) or (currItemStatus=fileStatusNotUnderSCC) then
            begin
            // Ignore - already checked in
            end
          else
            begin
            if not(FileBackup(singleItem)) then
              begin
              errMsg := 'This item could not be backed up prior to uncheckout';
              end;
            end;
          end;

        end;

    end;
      
    end;


  skipItem := FALSE;
  if (errMsg<>'') then
    begin
    allOK := (MessageDlg(
                         singleItem+CRLF+
                         CRLF+
                         errMsg+CRLF+
                         CRLF+
                         'Do you wish to ignore this item and continue processing?',
                        mtConfirmation,
                        [mbYes, mbNo],
                        0
                       ) = mrYes);

    dlgProgress.Cancel := not(allOK);
    skipItem := TRUE;
    end;



  if (allOK) then
    begin
    if ( (recursive) and DirectoryExists(singleItem) ) then
      begin
      if (skipItem) then
        begin
        increment := DeepCountItemsUnder(singleItem);
        end
      else
        begin
        fileIterator := TSDUFileIterator.Create(nil);
        try
          // Process the items under the directory
          fileIterator.Directory := singleItem;
          fileIterator.FileMask := '*.*';
          fileIterator.RecurseSubDirs := FALSE;
          fileIterator.OmitStartDirPrefix := FALSE;
          fileIterator.IncludeDirNames := TRUE;
          fileIterator.Reset();

          tmpFilename:= fileIterator.Next;
          while ((tmpFilename <> '') and (allOK)) do
            begin
            allOK := DoActualProcessSingleItem(
                                               operation,
                                               tmpFilename, recursive, dlgProgress,
                                               fileType,
                                               makeBackup,
                                               comment, keepCheckedOut
                                              );
            tmpFilename:= fileIterator.Next;
            end;

        finally
          fileIterator.Free();
        end;

        end;

      end;
    end;


  dlgProgress.IncPosition(increment);

  Result := allOK;
end;



function TContextMenu.DoActualProcessItems(
                                   operation: TInternalOp;
                                   items: TStringList; recursive: boolean; dlgProgress: TfrmProgress;
                                   // Only valid for opAdd
                                   fileType: TCCFileType;
                                   // Only valid for opUncheckout
                                   makeBackup: boolean;
                                   // Only valid for opCheckin
                                   comment: string; keepCheckedOut: boolean
                                  ): boolean;
var
  allOK: boolean;
  i: integer;
  dlgTitle: string;
begin
  case operation of
    opAddItems:
      begin
      dlgTitle := 'Adding items...';
      end;

    opCheckin:
      begin
      dlgTitle := 'Checkin items...';
      end;

    opCheckout:
      begin
      dlgTitle := 'Checkout items...';
      end;

    opUncheckout:
      begin
      dlgTitle := 'Uncheckout items...';
      end;

    internalBackup:
      begin
      dlgTitle := 'Backing up items before uncheckout...';
      end;

  end;

  dlgProgress.Caption := dlgTitle;
  

  allOK := TRUE;
  for i:=0 to (items.count-1) do
    begin
    allOK := DoActualProcessSingleItem(
                                       operation,
                                       items[i], recursive, dlgProgress,
                                       fileType,
                                       makeBackup,
                                       comment, keepCheckedOut
                                      );

    if not(allOK) then
      begin
      break;
      end;
    end;

  Result := allOK;
end;



function TContextMenu.FileBackup(srcFilename: string): boolean;
begin
  Result := FileCopy(srcFilename, GetBackupFilename(srcFilename));

end;


function TContextMenu.GetBackupFilename(srcFilename: string): string;
var
  x: integer;
  retval: string;
begin
  retval := '';

  // Identify a suitable filename
  x := 1;
  retval := srcFilename+'.preuncheckout';
  while (FileExists(retval)) do
    begin
    inc(x);
    retval := srcFilename+'.preuncheckout_'+inttostr(x);
    end;

  Result := retval;
end;


function TContextMenu.FileCopy(srcFilename, destFilename: string): boolean;
var
  fileOpStruct: TSHFileOpStruct;
begin
  // srcFilename and destFilename *MUST* end in a double NULL for
  // SHFileOperation to operate correctly
  srcFilename := srcFilename + #0 + #0;
  destFilename := destFilename + #0 + #0;
  fileOpStruct.Wnd:= 0;
  fileOpStruct.wFunc:= FO_COPY;
  fileOpStruct.pFrom:= PChar(srcFilename);
  fileOpStruct.pTo:= PChar(destFilename);
  fileOpStruct.fFlags:= (FOF_NOCONFIRMATION or FOF_NOCONFIRMMKDIR);
  fileOpStruct.fAnyOperationsAborted:= FALSE;
  fileOpStruct.hNameMappings:= nil;
  fileOpStruct.lpszProgressTitle:= nil;

  Result := (SHFileOperation(fileOpStruct) = 0);

end;


// This function will count the number of under, and including the specified
// item
function TContextMenu.DeepCountItemsUnder(item: string): integer;
var
  fileIterator: TSDUFileIterator;
  retval: integer;
begin
  retval := 1;

  if DirectoryExists(item) then
    begin
    fileIterator:= TSDUFileIterator.Create(nil);
    try
      fileIterator.Directory := item;
      fileIterator.FileMask := '*.*';
      fileIterator.RecurseSubDirs := FALSE;
      fileIterator.OmitStartDirPrefix := FALSE;
      fileIterator.IncludeDirNames := TRUE;
      fileIterator.Reset();

      retval := retval + fileIterator.Count();
    finally
      fileIterator.Free();
    end;

    end;

  Result := retval;  
end;


type
  TContextMenuFactory = class(TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;


// Register/Unregister this DLL
// Note: AppPath registration is assumed - done in the main app
procedure TContextMenuFactory.UpdateRegistry(Register: Boolean);
var
  ClassID: string;
  registry: TRegistry;
  registry2: TRegistry;
begin
  inherited UpdateRegistry(Register);

  registry := TRegistry.create();
  try
    registry.RootKey := HKEY_CLASSES_ROOT;
    try
      if Register then
        begin
        ClassID := GUIDToString(Class_ContextMenu);

        // All extensions
        registry.OpenKey(REGKEY_ALLEXT, TRUE);
        registry.WriteString('', ClassID);
        registry.CloseKey();

        // All directories
        registry.OpenKey(REGKEY_DIR, TRUE);
        registry.WriteString('', ClassID);
        registry.CloseKey();

        if (Win32Platform = VER_PLATFORM_WIN32_NT) then
          begin
          registry2:= TRegistry.Create();
          try
            registry2.RootKey := HKEY_LOCAL_MACHINE;
            registry2.OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Shell Extension', True);
            registry2.OpenKey('Approved', True);
            registry2.WriteString(ClassID, APP_TITLE);
          finally
            registry2.Free;
          end;
          
          end;
        end
      else
        begin
        registry.DeleteKey(REGKEY_ALLEXT);
        registry.DeleteKey(REGKEY_DIR);
// Drive not needed
//        registry.DeleteKey(REGKEY_DRIVE);
        end;

    except
      on ERegistryException do
        begin
        MessageDlg(
                   'An error occured while writing to the registry; if you are running under'+CRLF+
                   'NT, please ensure that you have the necessary permissions do perform'+CRLF+
                   'this operation (i.e. logon as administrator) and try again',
                   mtError,
                   [mbOK],
                   0
                  );
        end;
    end;

  finally
    registry.Free();
  end;

end;




initialization
  TContextMenuFactory.Create(ComServer,
                             TContextMenu,
                             Class_ContextMenu,
                             '',
                             APP_TITLE,
                             ciMultiInstance,
                             tmApartment);
END.


