unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls,
  ConsoleApp, CodeCoop, ExtCtrls, DirLabel, SDUFileIterator_U,
  SDUDirIterator_U;

type
  TForm1 = class(TForm)
    reReport: TRichEdit;
    Item: TGroupBox;
    Label2: TLabel;
    pbBrowse: TButton;
    edFilename: TEdit;
    pbFileStatus: TButton;
    pbCheckIn: TButton;
    pbCheckOut: TButton;
    pbAddFile: TButton;
    pbUncheckout: TButton;
    rgFileType: TRadioGroup;
    reComments: TRichEdit;
    ckKeepCheckedOut: TCheckBox;
    OpenDialog1: TOpenDialog;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    pbGetProjects: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure pbGetProjectsClick(Sender: TObject);
    procedure pbInitClick(Sender: TObject);
    procedure pbBrowseClick(Sender: TObject);
    procedure pbFileStatusClick(Sender: TObject);
    procedure pbAddFileClick(Sender: TObject);
    procedure pbCheckInClick(Sender: TObject);
    procedure pbCheckOutClick(Sender: TObject);
    procedure pbUncheckoutClick(Sender: TObject);
  private
    codeCoopObj: TCodeCoop;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  SDUGeneral;

procedure TForm1.FormShow(Sender: TObject);
begin
  codeCoopObj:= TCodeCoop.Create();
  codeCoopObj.Active := TRUE;
  reReport.Lines.Clear();

end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  codeCoopObj.Free();

end;

procedure TForm1.pbGetProjectsClick(Sender: TObject);
var
  name: TStringList;
  rootDir: TStringList;
  i: integer;
begin
  name:= TStringList.Create();
  try
    rootDir:= TStringList.Create();
    try

      if not(codeCoopObj.GetProjects(name, rootDir)) then
        begin
        reReport.lines.add('ERROR');
        end
      else
        begin
        reReport.lines.add('Projects available:');
        for i:=0 to (name.Count-1) do
          begin
          reReport.lines.add(inttostr(i)+' - '+name[i]+' -> '+rootDir[i]);
          end;
        end;

    finally
      rootDir.Free();
    end
  finally
    name.Free();
  end;
end;


procedure TForm1.pbInitClick(Sender: TObject);
begin
  codeCoopObj.Active := TRUE;

end;

procedure TForm1.pbBrowseClick(Sender: TObject);
begin
  SDUOpenSaveDialogSetup(OpenDialog1, edFilename.text);

  if OpenDialog1.execute then
    begin
    edFilename.text := OpenDialog1.Filename;
    end;

end;

procedure TForm1.pbFileStatusClick(Sender: TObject);
var
  fileStatus: TCCFileStatus;
  statusText: string;
begin
  fileStatus := codeCoopObj.FileStatus(edFilename.text);

  case fileStatus of
    fileStatusCheckedIn:
      begin
      statusText := 'Checked in';
      end;

    fileStatusCheckedOut:
      begin
      statusText := 'Checked out';
      end;

    fileStatusNotUnderSCC:
      begin
      statusText := 'Not under Code Co-op';
      end;

    fileStatusUnknown:
      begin
      statusText := 'UNKNOWN';
      end;

  end;

  reReport.lines.add(edFilename.text+' status: '+statusText);

end;


procedure TForm1.pbAddFileClick(Sender: TObject);
var
  fileType: TCCFileType;
begin
  case rgFileType.itemindex of
   0:
     begin
     fileType := fileTypeHeader;
     end;

   1:
     begin
     fileType := fileTypeSource;
     end;

   2:
     begin
     fileType := fileTypeText;
     end;

   else
     begin
     fileType := fileTypeBinary;
     end;

   end;


  if codeCoopObj.AddFile(edFilename.text, fileType) then
    begin
    reReport.lines.add('OK');
    end
  else
    begin
    reReport.lines.add('FAILED.');
    end;

end;

procedure TForm1.pbCheckInClick(Sender: TObject);
begin

  if codeCoopObj.CheckIn(edFilename.text, ckKeepCheckedOut.checked, reComments.Lines.text) then
    begin
    reReport.lines.add('OK');
    end
  else
    begin
    reReport.lines.add('FAILED.');
    end;

end;

procedure TForm1.pbCheckOutClick(Sender: TObject);
begin
  if codeCoopObj.CheckOut(edFilename.text) then
    begin
    reReport.lines.add('OK');
    end
  else
    begin
    reReport.lines.add('FAILED.');
    end;

end;

procedure TForm1.pbUncheckoutClick(Sender: TObject);
begin
  if codeCoopObj.UncheckOut(edFilename.text) then
    begin
    reReport.lines.add('OK');
    end
  else
    begin
    reReport.lines.add('FAILED.');
    end;

end;

END.


